/////////////////////////////////////////////////////////////////////////////

// Name:        ITest.h

// Purpose:     Declare abstract binary-decoupled interface for all tests.

// Author:      Marty Saxton

// Modified by:

// Created:     10/27/2001

// RCS-ID:      $Id: ITest.h,v 1.1.1.1 2001/11/10 19:00:42 thundercow Exp $

// Copyright:   (c) Marty Saxton, Thundercow Software

// Licence:     wxWindows license

/////////////////////////////////////////////////////////////////////////////



#ifndef __WXTEST_H__

#define __WXTEST_H__



#include <wxTest/preprocessor.h>



WXTEST_BEGIN_NAMESPACE



class wxITestRunner;



/** @defgroup Interfaces  Interfaces:  Pure abstract C++ classes that define the interfaces between tests and test-runners.

 */



/**

 * wxITest is the base interface for all tests.  This includes individual test cases and test suites.  

 * This class consists of nothing but pure virtual functions.

 * @ingroup Interfaces

 */

class wxITest 

{

public:



	/**

	 *  Destroys and deallocates the test, recursively if necessary.

	 */ 

	virtual void Delete() = 0;



	/**

	 *  Retreive a short human-readable description of this test.

	 *

	 *  @return A null terminated string that is the description of this test.

	 */ 

	virtual const char* GetDescription() = 0;



	/**

	 * The current state of this test.

	 * A test may be in one of the following states:  <br><br>

	 * <code>unknown</code>: the test hasn't yet been run<br>

	 * <code>running</code>: the test has been started and has not yet finished<br>

	 * <code>pass</code>: the test has completed and it passed<br>

	 * <code>fail</code>: the test has completed and it failed<br>

	 */ 

	enum TestState { unknown, running, pass, fail };



	/**

	 * Retreive the current TestState of this test.  See the TestState enum.

	 *

	 * @return The current TestState of this test.

	 */ 

	virtual TestState GetState() = 0;



	/**

	 * Assign a new state to this test.  This method is usually called by the test-runner after the test has completed.

	 *

	 * @return The current TestState of this test.

	 */ 

	virtual void SetState( TestState newState ) = 0;



	/**

	 * Called by the test-runner to begin execution of the test in a single-threaded context.  

	 *

	 * @param pResult The result object that will recieve the results of this test.

	 */ 

	virtual void Run( wxITestRunner* pResult ) = 0;



	/**

	 * Called by the test-runner to begin execution of the test in a multi-threaded context.

	 *

	 * @param pResult The result object that will recieve the results of this test.

	 * @param nThreadId The thread identifer of the thread in which this test is running.

	 */ 

	virtual void Run( wxITestRunner* pResult, long unsigned nThreadId ) = 0;



	/**

	 * Enables the test-runner to determine if this test is intended to be thread-safe.  

	 * This useful for test-runners that may want to run the test in multiple threads 

	 * simultaneously to test for resource conflicts and deadlocks.

	 */ 

	virtual bool IsThreadSafe() = 0;



	/**

	 * Retreive the number of sub-tests for this test.

	 * 

	 */ 

	virtual int GetNumSubTests() = 0;



	/**

	 * Retreive the child-tests of this test.

	 *

	 * @param pOutTests A pointer to a buffer that will recieve pointers to the sub-tests.

	 */ 

	virtual void GetSubTests( wxITest ** pOutTests ) = 0;



	/**

	 * Retreive the parent of this test.

	 *

	 * @return A pointer to the parent test of this test, or NULL if this test has no parent.

	 */ 

	virtual const wxITest* GetParent() const = 0;

	virtual wxITest* GetParent()  = 0;



	/**

	 * Assign a new parent to this test.

	 *

	 * @param pParent The test that is to be the parent of this test.

	 */ 

	virtual void SetParent( wxITest* pParent )  = 0;

};





/**

 * A wxTestAssertion is thrown whenever a failure occurs in a test 

 * (e.g., WXTASSERT failure or some other kind of exception).

 * This way if a failure occurs in the middle of a test, the remainder of 

 * code in that test is skipped.

 *

 * The wxTestAssertion object has a message, derived from the point of failure.

 */

class wxTestAssertion

{

public:

	/**

	 * Construct a new wxTestAssertion object.  This isn't a pure interface as it contains code,

	 * but since it's inline it won't create any link-time dependencies, and it's completely 

	 * self-contained (it doesn't use any external functions).

	 *

	 * @param pszMessage  The message that describes this exception.

	 * @param file        The source file at which this failure occured (if any).

	 * @param lineNo      The line in the code at which this failure occured (if any).

	 */

	wxTestAssertion( const char* pszMessage, const char* file, long unsigned lineNo ) :

		m_pszFile( file ), m_nLineNo( lineNo )

	{

		const char* pSrc = pszMessage;

		char* pDst = m_szMessage;

		while ( *pSrc )

			*pDst++ = *pSrc++;

		*pDst = 0;

	}



	char m_szMessage[1024];

	const char* m_pszFile;

	long unsigned m_nLineNo;

};



/**

 *  WXTASSERT evalueates a boolean expression.  

 *  A FALSE expression is considered a failure and a wxTestAssertion is thrown, 

 *  which must be caught by the test-runner.

 */



#define WXTASSERT( BOOLEXPR ) if ( !( BOOLEXPR ) ) throw wxTestAssertion( "", __FILE__, __LINE__ );



/**  

 *  WXTASSERT evalueates a boolean expression.  

 *  A FALSE expression is considered a failure and a wxTestAssertion is thrown, 

 *  which must be caught by the test-runner.

 */

#define WXTASSERTMSG( MSG, BOOLEXPR ) if ( !( BOOLEXPR ) ) throw wxTestAssertion( MSG, __FILE__, __LINE__ );



/** @)

 */





WXTEST_END_NAMESPACE



#endif //__WXTEST_H__

