/////////////////////////////////////////////////////////////////////////////

// Name:        ITestRunner.h

// Purpose:     

// Author:      Marty Saxton

// Modified by:

// Created:     10/27/2001

// RCS-ID:      $Id: ITestRunner.h,v 1.1.1.1 2001/11/10 19:00:42 thundercow Exp $

// Copyright:   (c) Marty Saxton, Thundercow Software

// Licence:     wxWindows license

/////////////////////////////////////////////////////////////////////////////



#ifndef __WXTESTRUNNER_H__

#define __WXTESTRUNNER_H__

class wxITest;



/**

 * The pure abstract interface for all test-runners.

 * This class consists of nothing but pure virtual functions.

 * @ingroup Interfaces

 */

class wxITestRunner

{

public:

	/**

	 * Implementations of test-runners can implement this function to spawn additional threads 

	 * to run test cases in a multi-threaded envoriment to check for thread saftey.

	 * This function is typically called from an a test's 'Run( wxITestRunner* )' function.

	 * It should spawn multiple threads and call the test's 'Run( wxITestRunner*, int nThreadId )' function

	 * 

	 * @param wxITest The test currently running.

	 */

	virtual void SpawnThreads( wxITest* ) = 0;



	/**

	 * Captures textual outputfrom tests.

	 * If tests create any output then it must be identical each

	 * time the test is run, as explained below.

	 * 

	 * The first time a test is run, the test-runner will

	 * prompt a user to confirm the correctness of the output.  Once test output has been

	 * confirmed as correct, the test-runner saves the output for later comparison.  Upon subsequent

	 * runs of the test, the confirmed output that has been saved is automatically compared against the new output.  

	 * If the output is not identical then the test is considered to have failed.

	 */

	virtual void wxtstdout( const char* psz, ... ) = 0;



	/**

	 * Called by the test framework before a unit test is run.  This function is overridden in specialized 

	 * implementation of test-runners to perform specific tasks (e.g., to update the test runner's user interface or 

	 * write to a log).

	 * @param pTest The test that about to run.

	 */

	virtual void BeginTestCase( wxITest* pTest ) = 0;



	/**

	 * Called by the test framework after a unit test is run. This function is overridden in specialized 

	 * implementation of test-runners to perform specific tasks (e.g., to update the test runner's user interface or 

	 * write to a log).

	 * @param pTest The test that just finnished running.

	 */

	virtual void EndTestCase( wxITest* pTest ) = 0;



	/**

	 * Called by the test framework when a test fails.  This function is overridden in specialized 

	 * implementation of test-runners to perform specific tasks (e.g., to update the test runner's user interface or 

	 * write to a log).

	 * @param pTest       The test that's currently running.

     * @param pszMessage  A human-readable message that describes the failure.

	 * @param pszFile     The source file in which the failure occured.

	 * @param nLineNo     The line number at which the failure occured.

	 */

	virtual void OnTestFailure( wxITest* pTest, const char* pszMessage, const char* pszFile, unsigned long nLineNo ) = 0; 



	/**

	 * Retreives the number of individual test-cases run thus far.

	 * @return The number of individual test-cases run thus far.

	 */

	virtual int GetNumTestsRun() = 0;



	/**

	 * Retreives the number of failures that have occured thus far.

	 * @return The number of faulures that have occured thus far.

	 */

	virtual int GetNumFailures() = 0;



};

#endif //__WXTESTRUNNER_H__

