/////////////////////////////////////////////////////////////////////////////

// Name:        TestCaller.h

// Purpose:     

// Author:      Marty Saxton

// Modified by:

// Created:     10/27/2001

// RCS-ID:      $Id: TestCaller.h,v 1.1.1.1 2001/11/10 19:00:42 thundercow Exp $

// Copyright:   (c) Marty Saxton, Thundercow Software

// Licence:     wxWindows license

/////////////////////////////////////////////////////////////////////////////





#ifndef CPPUNIT_TESTCALLER_H

#define CPPUNIT_TESTCALLER_H



#include "guards.h"

#include "TestCaseMultiThreaded.h"



/* 

 * A test caller provides access to a test case method

 * on a test case class.  Test callers are useful when

 * you want to run an individual test or add it to a

 * suite.

 *

 * Here is an example:

 *

 * class MathTest : public TestCase {

 *         ...

 *     public:

 *         void         setUp ();

 *         void         tearDown ();

 *

 *         void         testAdd ();

 *         void         testSubtract ();

 * };

 *

 * Test *MathTest::suite () {

 *     TestSuite *suite = new TestSuite;

 *

 *     suite->addTest (new wxTestCaller<MathTest> ("testAdd", testAdd));

 *     return suite;

 * }

 *

 * You can use a wxTestCaller to bind any test method on a TestCase

 * class, as long as it returns accepts void and returns void.

 *

 * See TestCase

 */



template <class Fixture> class wxTestCaller : public wxTestCaseMultiThreaded

{

	REFERENCEOBJECT (wxTestCaller);

	typedef void (Fixture::*TestMethod)( wxITestRunner* , long unsigned );



	public:

		wxTestCaller (const char* name, TestMethod test, wxITest* pParent = 0 ) :

			wxTestCaseMultiThreaded (name), 

			m_pFixture (new Fixture (name)), 

			m_state( unknown ),

			m_test (test)

		{

		}



	virtual ~wxTestCaller()

	{

		delete m_pFixture;

	}





	protected:

		void runTest ()

		{ 

			(m_pFixture.get ()->*m_test)(); 

		}









	private:







	/**

	 *  Destroys and deallocates the test

	 */ 

	virtual void Delete() { delete this; }



	/**

	 *  Retreive a human readable description of this test.

	 *

	 *  @return A null terminated string that is the description of this test.

	 */ 

	virtual const char* GetDescription() { return m_pFixture->GetDescription(); }



	/**

	 * Retreive the current TestState of this test.

	 *

	 * @return The current TestState of this test.

	 */ 

	virtual TestState GetState() { return m_state; }



	virtual void SetState( TestState newState ) { m_state = newState; }



	/**

	 * Determine if this test is intended to be thread-safe.  This useful for test runners

	 * that may want to run the test in multiple threads simultaneously to test for 

	 * resource conflicts and deadlocks.

	 */ 

	virtual bool IsThreadSafe() { return m_pFixture->IsThreadSafe(); }



	/**

	 * Retreive the number of sub-tests for this test.

	 * 

	 */ 

	virtual int GetNumSubTests() { return m_pFixture->GetNumSubTests(); }



	/**

	 * Retreive the child-tests of this test.

	 *

	 * @param pOutTests A pointer to a buffer that will recieve pointers to the sub-tests.

	 */ 

	virtual void GetSubTests( wxITest ** pOutTests ) { m_pFixture->GetSubTests(pOutTests); }



	/**

	 * Retreive the parent of this test.

	 *

	 * @return A pointer to the parent test of this test, or NULL if this test has no parent.

	 */ 

	virtual const wxITest* GetParent() const  { return m_pFixture->GetParent(); }



	/**

	 * Retreive the parent of this test.

	 *

	 * @return A pointer to the parent test of this test, or NULL if this test has no parent.

	 */ 

	virtual wxITest* GetParent() { return m_pFixture->GetParent(); }



	/**

	 * Assign a new parent to this test.

	 *

	 * @param pParent The test that is to be the parent of this test.

	 */ 

	virtual void SetParent( wxITest* pParent ) { m_pFixture->SetParent( pParent ); }



	/**

	 * Set up this object's state before actually executing any tests.

	 */

	virtual void Setup() { m_pFixture->Setup(); }



	/**

	 * Clean up anything that was allocated in Setup.

	 */

	virtual void TearDown() { m_pFixture->TearDown(); }



protected:

	virtual void RunUnitTest( wxITestRunner* pResult, long unsigned nThreadId ) { (m_pFixture->*m_test)( pResult, nThreadId ); };



private:

	TestMethod m_test;

	Fixture* m_pFixture;

	TestState m_state;

};

#endif

